/*
Take an error message log on stdin with lines of the form

Warning file linenumber : error message
Error file linenumber : error message

and display on stdout the error message followed by the offending
source lines.

Turbo C uses this format for its messages.

Usage:
	terrshow <error_log_file >merged_source_and_error_file

[27-Feb-88]
*/

/* This version works with Turbo C Version 1.5 or later */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define MAXLINE 255
#define MAXFNAME 63	/* PC DOS has short names */

#define SUCCESS 0
#define FAILURE 1

char errmsg[MAXLINE];
char line[MAXLINE];
char fname[MAXFNAME] = "";
FILE *fp = (FILE *)NULL;
int lno = 0;
int errlno = 0;

void cleanup();
void display();
int getfile();

void
main()
{
	while (fgets(errmsg,MAXLINE,stdin) != (char *)NULL)
		(void)display();
	(void)cleanup();
}

void
cleanup()	/* clean up and exit */
{
	if (fp != (FILE *)NULL)
		(void)fclose(fp);
	exit(0);
}

void
display()	/* display source line matching errmsg[] */
{
	char *p;
	char *plno;
	char *perr;
	
	if ((strncmp(errmsg,"Warning ",8) != 0) &&
		(strncmp(errmsg,"Error ",6) != 0))
		return;	/* not an error message line */
	p = strchr(errmsg,' ');
	*p++ = '\0';	/* clobber ' '; p points now to file name */
	plno = strchr(p,' ');
	*plno++ = '\0';		/* point to line number */
	perr = strchr(plno,':');
	*perr = '\0';		/* terminate line number */
	perr += 2;		/* point to error message text */
	if ((errlno = atoi(plno)) <= 0)	/* error line number in source file */
		return;			/* not an error message line */
	if (getfile(p) != SUCCESS)
		return;
	if (lno > errlno)	/* cannot happen */
	{
		printf("\n?Line number backed up: %s(%s\n",errmsg,p);
		rewind(fp);
		lno = 0;
	}
	while (lno < errlno)
	{
		if (fgets(line,MAXLINE,fp) == (char *)NULL)
		{
			printf("\n?Unexpected EOF before finding line: %s(%s\n",
				errmsg,p);
			return;
		}
		lno++;
	}
	printf("\n%s(%s:%7d)%s",perr,p,lno,line);
}

int
getfile(errmsg)	/* return SUCCESS on success, FAILURE on error */
char *errmsg;
{
	if (strcmp(fname,errmsg) != 0)
	{
		if (fp != (FILE *)NULL)
			(void)fclose(fp);
		(void)strncpy(fname,errmsg,MAXFNAME);
		fname[MAXFNAME-1] = '\0';
		if ((fp = fopen(fname,"r")) == (FILE *)NULL)
		{
			printf("\n?Cannot open file [%s]\n",fname);
			return(FAILURE);
		}
		lno = 0;		
	}
	return(SUCCESS);
}
